@@ -50,6 +50,14 @@ static void handle_output_leave(
 	}
 }
 
+static void handle_destroy(
+		struct wl_listener *listener, void *data) {
+	struct sway_container *con = wl_container_of(
+			listener, con, output_handler_destroy);
+
+	container_begin_destroy(con);
+}
+
 static bool handle_point_accepts_input(
 		struct wlr_scene_buffer *buffer, double *x, double *y) {
 	return false;
@@ -135,6 +143,9 @@ struct sway_container *container_create(struct sway_view *view) {
 			c->output_leave.notify = handle_output_leave;
 			wl_signal_add(&c->output_handler->events.output_leave,
 					&c->output_leave);
+			c->output_handler_destroy.notify = handle_destroy;
+			wl_signal_add(&c->output_handler->node.events.destroy,
+					&c->output_handler_destroy);
 			c->output_handler->point_accepts_input = handle_point_accepts_input;
 		}
 	}
@@ -550,6 +561,12 @@ void container_begin_destroy(struct sway_container *con) {
 	if (con->pending.parent || con->pending.workspace) {
 		container_detach(con);
 	}
+
+	if (con->view && con->view->container == con) {
+		wl_list_remove(&con->output_enter.link);
+		wl_list_remove(&con->output_leave.link);
+		wl_list_remove(&con->output_handler_destroy.link);
+	}
 }
 
 void container_reap_empty(struct sway_container *con) {
@@ -694,26 +711,35 @@ size_t parse_title_format(struct sway_container *container, char *buffer) {
 		len += next - format;
 		format = next;
 
-		if (strncmp(next, "%title", 6) == 0) {
+		if (has_prefix(next, "%title")) {
 			if (container->view) {
 				len += append_prop(buffer, view_get_title(container->view));
 			} else {
 				len += container_build_representation(container->pending.layout, container->pending.children, buffer);
 			}
-			format += 6;
+			format += strlen("%title");
 		} else if (container->view) {
-			if (strncmp(next, "%app_id", 7) == 0) {
+			if (has_prefix(next, "%app_id")) {
 				len += append_prop(buffer, view_get_app_id(container->view));
-				format += 7;
-			} else if (strncmp(next, "%class", 6) == 0) {
+				format += strlen("%app_id");
+			} else if (has_prefix(next, "%class")) {
 				len += append_prop(buffer, view_get_class(container->view));
-				format += 6;
-			} else if (strncmp(next, "%instance", 9) == 0) {
+				format += strlen("%class");
+			} else if (has_prefix(next, "%instance")) {
 				len += append_prop(buffer, view_get_instance(container->view));
-				format += 9;
-			} else if (strncmp(next, "%shell", 6) == 0) {
+				format += strlen("%instance");
+			} else if (has_prefix(next, "%shell")) {
 				len += append_prop(buffer, view_get_shell(container->view));
-				format += 6;
+				format += strlen("%shell");
+			} else if (has_prefix(next, "%sandbox_engine")) {
+				len += append_prop(buffer, view_get_sandbox_engine(container->view));
+				format += strlen("%sandbox_engine");
+			} else if (has_prefix(next, "%sandbox_app_id")) {
+				len += append_prop(buffer, view_get_sandbox_app_id(container->view));
+				format += strlen("%sandbox_app_id");
+			} else if (has_prefix(next, "%sandbox_instance_id")) {
+				len += append_prop(buffer, view_get_sandbox_instance_id(container->view));
+				format += strlen("%sandbox_instance_id");
 			} else {
 				lenient_strcat(buffer, "%");
 				++format;
@@ -776,7 +802,7 @@ size_t container_build_representation(enum sway_container_layout layout,
 			len += strlen(identifier);
 			lenient_strcat(buffer, identifier);
 		} else {
-			len += 6;
+			len += strlen("(null)");
 			lenient_strcat(buffer, "(null)");
 		}
 	}
